#!/usr/bin/env ruby

=begin
WPScan - WordPress Security Scanner
Copyright (C) 2011  Ryan Dewhurst AKA ethicalhack3r

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ryandewhurst at gmail
=end

# This library should contain all methods used to gather (discover) information
# from a wordpress installation.

class Discover

  attr_accessor :url, :threads, :verbose

  def initialize(url, threads, verbose)
    @url = url
    @threads = threads
    @verbose = verbose
  end

  # Checks to see if the readme.html file exists
  #
  # This file comes by default in a wordpress installation,
  # and if deleted is reinstated with an upgrade.

  def readme_exists?

    exists = false

    response = Typhoeus::Request.get(@url.to_s + 'readme.html')
    
    if response.code == 200
      if response.body =~ %r{Semantic Personal Publishing Platform}i
        exists = true
      end
    end

    exists

  end

  # Attempts to find the wordpress version from,
  # the generator meta tag in the html source.
  #
  # The meta tag can be removed however it seems, 
  # that it is reinstated on upgrade.

  def generator_version

    version = false

    response = Typhoeus::Request.get(@url.to_s)

    # follow redirects...

    until response.code !~ /30/
      response = Net::HTTP.get_response(URI.parse(response.headers_hash['location']))
    end

    version = response.body[%r{name="generator" content="wordpress (.*)"}i, 1]

  end

  # Enumerate wordpress usernames by using Veronica Valeros's technique:
  # http://seclists.org/fulldisclosure/2011/May/493 

  def usernames

    usernames = []
   
    # Iterate through 10 wordpress user ids

    (1..10).each do |author_count|

      # build the new URL and send the request

      response = Typhoeus::Request.get(@url.to_s + '?author=' + author_count.to_s)

      if response.code == 301 # username in location? 
        usernames.push(response.headers_hash['location'][%r{/author/(.*)/}i, 1])
      elsif response.code == 200 # username in body?
        usernames.push(response.body[%r{posts by (.*) feed}i, 1])
      end

    end

    # clean the array, remove nils and possible duplicates

    usernames.flatten!
    usernames.compact!
    usernames.uniq!

    return usernames

  end

  # Compare the wordpress version to a list of vulnerabilities,
  # the vulnerability file is stored in data/wp_vuln.xml.

  def wp_vulnerabilities(version)
    data = XmlSimple.xml_in('data/wp_vulns.xml', { 'KeyAttr' => 'version' })
    vulnerabilities = data['wordpress'][version] 
  end

  # See if there are any vulnerabilities associated with the,
  # plugins which we found to be installed.    

  def plugin_vulnerabilities(plugins)
    vulnerabilities = []
    data = XmlSimple.xml_in('data/plugin_vulns.xml', { 'KeyAttr' => 'plugin' })
    data['plugin'].each do |plugin|
      if plugins.include?(plugin['name'])
        vulnerabilities << plugin
      end
    end
    vulnerabilities
  end

  # Discover the wordpress theme name by parsing the csss link rel

  def theme_name

    response = Typhoeus::Request.get(@url.to_s)

    # follow redirects...
    until response.code !~ /30/
      response = Net::HTTP.get_response(URI.parse(response.headers_hash['location']))
    end

    theme = response.body[%r{/wp-content/themes/(.*)/style.css}i, 1]
  end

  # Discover any error_log files created by WordPress
  # These are created by the WordPress error_log() function
  # They are normally found in the /plugins/ directory,
  # however can be found in their specific plugin dir.
  # http://www.exploit-db.com/ghdb/3714/

  def error_log

    exists = false

    response = Typhoeus::Request.get(@url.to_s + 'wp-content/plugins/error_log')

    if response.code == 200
      if response.body =~ %r{PHP Fatal error}i
        exists = true
      end
    end

    exists

  end

  # Enumerate installed plugins.

  def plugins

    hydra = Typhoeus::Hydra.new(:max_concurrency => @threads, :timeout => 2000)

    found_plugins = []
    request_count = 0
    queue_count = 0

    # Open and parse the 'most popular' plugin list...
    file_contents = File.open("data/plugins.txt","r") {|file| file.readlines.collect{|line| line.chomp}}

    # Open and parse the plugins we know have vulnerabilities...
    xml_contents = XmlSimple.xml_in('data/plugin_vulns.xml', { 'KeyAttr' => 'name' })

    # Join both plugin arrays and make them into a set
    plugins = file_contents | xml_contents['plugin'].keys

    puts
    puts "Checking for " + plugins.size.to_s + " plugins..." 

    plugins.each do |plugin|

      request = Typhoeus::Request.new(@url.to_s + "wp-content/plugins/" + plugin + "/")
      request_count += 1      

      request.on_complete do |response|
        if response.code == 200 or response.code == 403
          found_plugins.push(plugin)
        end
      end

      hydra.queue(request)
      queue_count += 1

      if queue_count == @threads
        hydra.run
        queue_count = 0
      end

    end

    hydra.run

    found_plugins

  end

  # Is directory listing enabled?
  # WordPress denies directory listing however,
  # forgets about the plugin directory.

  def directory_listing?(plugin)

    listing = false

    response = Typhoeus::Request.get(@url.to_s + 'wp-content/plugins/' + plugin + '/')

    if response.body[%r{<title>Index of}]
      listing = true
    end

    listing

  end

  # Checks if the 'login lockdown' plugin is enabled
  # Thanks to Alip Aswalid for providing this method.

  def login_lockdown
  
    exists = false
    
    response = Typhoeus::Request.get(@url.to_s + 'wp-login.php')

    if response.code == 200
      if response.body =~ %r{Login LockDown}i
        exists = true
      end
    end

    exists

  end


end
